/*
*    (descriptor.c)
*/

#include "kernel.h"


void setGlobalDescriptor (int dsc, address base, unsigned int limit, int dpl, int sbit, int type);
void setIntDescriptor    (int dsc, short int cs, address offset, int info);



/*
*  Function: initDescriptors
*
*    GDT/IDTディスクリプタを初期化する
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    成功した場合はtrue、失敗した場合はfalseを返します。
*
*/

bool initDescriptors (void){


   /*
    *  GDTを作成
    */

    for(int i = 0; i < 8192; i++)
        setGlobalDescriptor(i, 0, 0, 0, 0, 0);

    setGlobalDescriptor(GDT_CODE_SELECTOR,      BOOT3_ADDRESS,  0xfffff, 0, 1, GDTYPE_CODE);
    setGlobalDescriptor(GDT_DATA_SELECTOR,            0x00000,  0xfffff, 0, 1, GDTYPE_DATA);
    setGlobalDescriptor(GDT_TSS_SELECTOR,         TSS_ADDRESS,     0x68, 0, 0, GDTYPE_TSS);
    setGlobalDescriptor(GDT_USERCODE_SELECTOR,              0,  0xfffff, 0, 1, GDTYPE_CODE+0x60);
    setGlobalDescriptor(GDT_USERDATA_SELECTOR,              0,  0xfffff, 0, 1, GDTYPE_DATA+0x60);



   /*
    *  IDTを作成
    */

    // 例外ハンドラ
    setIntDescriptor(0,  GDT_CODE_SELECTOR, (address) &asm_exception0-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(1,  GDT_CODE_SELECTOR, (address) &asm_exception1-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(2,  GDT_CODE_SELECTOR, (address) &asm_exception2-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(3,  GDT_CODE_SELECTOR, (address) &asm_exception3-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(4,  GDT_CODE_SELECTOR, (address) &asm_exception4-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(5,  GDT_CODE_SELECTOR, (address) &asm_exception5-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(6,  GDT_CODE_SELECTOR, (address) &asm_exception6-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(7,  GDT_CODE_SELECTOR, (address) &asm_exception7-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(8,  GDT_CODE_SELECTOR, (address) &asm_exception8-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(9,  GDT_CODE_SELECTOR, (address) &asm_exception9-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(10, GDT_CODE_SELECTOR, (address) &asm_exception10-BOOT3_ADDRESS, IDT_INT_HANDLER);
    setIntDescriptor(11, GDT_CODE_SELECTOR, (address) &asm_exception11-BOOT3_ADDRESS, IDT_INT_HANDLER);
    setIntDescriptor(12, GDT_CODE_SELECTOR, (address) &asm_exception12-BOOT3_ADDRESS, IDT_INT_HANDLER);
    setIntDescriptor(13, GDT_CODE_SELECTOR, (address) &asm_exception13-BOOT3_ADDRESS, IDT_INT_HANDLER);
    setIntDescriptor(14, GDT_CODE_SELECTOR, (address) &asm_exception14-BOOT3_ADDRESS, IDT_INT_HANDLER);
    setIntDescriptor(15, GDT_CODE_SELECTOR, (address) &asm_exception15-BOOT3_ADDRESS, IDT_INT_HANDLER);
    setIntDescriptor(16, GDT_CODE_SELECTOR, (address) &asm_exception16-BOOT3_ADDRESS, IDT_INT_HANDLER);
    setIntDescriptor(17, GDT_CODE_SELECTOR, (address) &asm_exception17-BOOT3_ADDRESS, IDT_INT_HANDLER);
    setIntDescriptor(18, GDT_CODE_SELECTOR, (address) &asm_exception18-BOOT3_ADDRESS, IDT_INT_HANDLER);
    setIntDescriptor(19, GDT_CODE_SELECTOR, (address) &asm_exception19-BOOT3_ADDRESS, IDT_INT_HANDLER);



    // 割り込みハンドラ
    setIntDescriptor(IDT_TIMER_SELECTOR,      GDT_CODE_SELECTOR, (address) &asm_inthd0-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(IDT_KEYBOARD_SELECTOR,   GDT_CODE_SELECTOR, (address) &asm_inthd1-BOOT3_ADDRESS,  IDT_INT_HANDLER);
    setIntDescriptor(IDT_MOUSE_SELECTOR,      GDT_CODE_SELECTOR, (address) &asm_inthd12-BOOT3_ADDRESS, IDT_INT_HANDLER);

    // TEST
    setIntDescriptor(IDT_ELFTEST_SELECTOR,      GDT_CODE_SELECTOR, (address) &asm_elfhd-BOOT3_ADDRESS, IDT_INT_HANDLER+0x60);


    // LDTRとGDTRをする

    *((short int *) LDTR_ADDRESS)         = 2047;
    *((int *) ((void *) LDTR_ADDRESS+2))  = IDT_ADDRESS;
    lidt();

    *((short int *) GDTR_ADDRESS)        = 0xffff;
    *((int *) ((void *) GDTR_ADDRESS+2)) = GDT_ADDRESS;
    lgdt();

    return true;
}



/*
*  Function: setGlobalDescriptor
*
*    グローバルディスクリプタを設定する
*
*  Parameters:
*
*    dsc   -  ディスクリプタの番号
*    base  -  ベースアドレス
*    limit -  上限値
*    dpl   -  DPL Flagをセットするか
*    sbit  -  S Flagをセットするか
*    type  -  ディスクリプタの種類
*
*  Returns:
*
*    なし
*
*/

void setGlobalDescriptor (int dsc, address base, unsigned int limit, int dpl, int sbit, int type){


 GlobalDescriptor *descriptor = (GlobalDescriptor *) GDT_ADDRESS+(dsc/8);

  if(!type){
    descriptor->limit1 = 0;
    descriptor->limit2 = 0;
    descriptor->base1  = 0;
    descriptor->base2  = 0;
    descriptor->base3  = 0;
    descriptor->info   = 0;
    return;
  }


  descriptor->limit1   = limit&0xffff;
  descriptor->limit2   = 0xcf;
  descriptor->base1    = base&0xffff;
  descriptor->base2    = (base>>16)&0xff;
  descriptor->base3    = (base>>24)&0xff;
  descriptor->info     = 0x80|(dpl<<5)|(sbit<<4)|type;

  return;
}




/*
*  Function: setIntDescriptor
*
*    割り込みディスクリプタを設定する
*
*  Parameters:
*
*    dsc    -  ディスクリプタの番号
*    cs     -  割り込みハンドラのコードセグメントディスクリプタ
*    offset -  割り込みハンドラのオフセット
*    info   -  ディスクリプタの設定
*
*  Returns:
*
*    なし
*
*/

void setIntDescriptor (int dsc, short int cs, address offset, int info){


 IntDescriptor *descriptor = (IntDescriptor *) IDT_ADDRESS+dsc;

  descriptor->offset1  = offset&0xffff;
  descriptor->offset2  = offset>>16;
  descriptor->cs       = cs;
  descriptor->info     = info;
  descriptor->unused   = 0;

  return;
}

