/*
 *   (fs.c)
 */

#include "kernel.h"


#define RESFS_VERSION            1
#define RESFS_SIGNATURE_1        0xa0
#define RESFS_SIGNATURE_2        "ResFS"
#define RESFS_COMPATIBILITY_YES  0xcc
#define RESFS_COMPATIBILITY_NO   0xff
#define RESFS_FLAG_USING         1
#define RESFS_FILES_NUM          128


struct Info{
 char signature[6];
 char version;
 char compatibility;
 unsigned int dataAddress;
};

struct File{
 unsigned int flag;
 char filename[32];
 unsigned int address;
 unsigned int size;
};

struct ResFS{
 struct Info  INFO;
 struct File  FILES[RESFS_FILES_NUM];
};



struct ResFS *fs;




/*
*  Function: initFS
*
*    ファイルシステムを初期化する
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    成功した場合はtrue、失敗した場合はfalseを返します。
*
*/

bool initFS (void){

  fs = (struct ResFS *) ((void *) *((unsigned int *) 0x7c03)+BOOT3_ADDRESS-0x600);
  return true;
}




/*
*  Function: fs_searchEntry
*
*    ファイル名からファイルのIDを取得する
*
*  Parameters:
*
*    filename -  ファイル名
*
*  Returns:
*
*    成功した場合はファイルのID、失敗した場合は0xffffffffを返します。
*
*/

unsigned int fs_searchEntry (const char *filename){

 unsigned int i;

  for(i=0; (strcmp(fs->FILES[i].filename, filename) != 0) && (i < RESFS_FILES_NUM); i++);

  if(i == RESFS_FILES_NUM)
   return 0xffffffff;
  else
   return i;
}




/*
*  Function: fs_size
*
*    ファイル名からファイルサイズを取得する
*
*  Parameters:
*
*    filename -  ファイル名
*
*  Returns:
*
*    ファイルサイズ(バイト単位)を返します。
*
*/

size fs_size (const char *filename){
  return fs->FILES[fs_searchEntry(filename)].size;
}





/*
*  Function: fs_load
*
*    ファイルを読み込む
*
*  Parameters:
*
*    address  -  ファイルを書き込むアドレス
*    filename -  ファイル名
*
*  Returns:
*
*    成功した場合はtrue、失敗した場合はfalseを返します。
*
*/

bool fs_load (char *address, char *filename){

 char *p;
 unsigned int i, j, size;

  if((i = fs_searchEntry(filename)) == 0xfffffff)
    return false;

  size = fs_size(filename);
  p    = (char*) fs+fs->INFO.dataAddress+fs->FILES[i].address;

  for(j=0; (j < size); j++, p++){
   address[j] = *p;
  }

  return true;
}




/*
*  Function: fileLoad
*
*    ファイルを読み込む (fs_load関数のエイリアスです)
*
*  Parameters:
*
*    address  -  ファイルを書き込むアドレス
*    filename -  ファイル名
*
*  Returns:
*
*    成功した場合はtrue、失敗した場合はfalseを返します。
*
*/

bool fileLoad   (void *address, char *filename){

  return fs_load((char *) address, filename);
}




/*
*  Function: fs_searchEntry
*
*    ファイルが存在するか調べる
*
*  Parameters:
*
*    filename -  ファイル名
*
*  Returns:
*
*    存在する場合はtrue、存在しない場合はfalseを返します。
*
*/

bool fileExists (const char *filename){

  if(fs_searchEntry(filename) == 0xffffffff)
    return false;
  else
    return true;
}




/*
*  Function: fileSize
*
*    ファイル名からファイルサイズを取得する (fs_load関数のエイリアスです)
*
*  Parameters:
*
*    filename -  ファイル名
*
*  Returns:
*
*    ファイルサイズ(バイト単位)を返します。
*
*/

size fileSize (const char *filename){

  return fs_size(filename);
}

