/*
 *   (keyboard.c)
 */

#include "kernel.h"

char convKeycode (unsigned char keycode);
void keyboardLED (unsigned char target);
void waitKBC (int bit);


//キーコード
#define KEY_ESC         0x01
#define KEY_BACKSPACE   0x0e
#define KEY_TAB         0x0f
#define KEY_ENTER       0x1c
#define KEY_HANKAKU     0x29
#define KEY_HIRAGANA    0x70
#define KEY_SPACE       0x39
#define KEY_SYSREQ      0x54
#define KEY_ALT         0x38
#define KEY_CTRL        0x1d
#define KEY_R_SHIFT     0x36
#define KEY_L_SHIFT     0x2a
#define KEY_CAPSLOCK    0x3a
#define KEY_NUMLOCK     0x45
#define KEY_SCROLLLOCK  0x46
#define KEY_F1          0x3b
#define KEY_F2          0x3c
#define KEY_F3          0x3d
#define KEY_F4          0x3e
#define KEY_F5          0x3f
#define KEY_F6          0x40
#define KEY_F7          0x41
#define KEY_F8          0x42
#define KEY_F9          0x43
#define KEY_F10         0x44
#define KEY_F11         0x57
#define KEY_F12         0x58


static unsigned char kbmap1[] = {
    0, KEY_ESC, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '^', KEY_BACKSPACE,
    KEY_TAB, 'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', '@', '[', KEY_ENTER, KEY_CTRL,
    'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ';', ':', KEY_HANKAKU, KEY_L_SHIFT, ']',
    'Z', 'X', 'C', 'V', 'B', 'N', 'M', ',', '.', '/', KEY_R_SHIFT, '*',
    KEY_ALT, KEY_SPACE, KEY_CAPSLOCK, KEY_F1, KEY_F2, KEY_F3, KEY_F4,
    KEY_F5, KEY_F6, KEY_F7, KEY_F8, KEY_F9, KEY_F10, KEY_NUMLOCK, KEY_SCROLLLOCK,
    '7', '8', '9', '-', '4', '5', '6', '+', '1', '2', '3', '0', '.', KEY_SYSREQ, 0, 0, KEY_F11, KEY_F12,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    KEY_HIRAGANA, 0, 0, '_', 0, 0, 0, 0, 0, 0, 0, 0, 0, '\\', 0, 0
};

static unsigned char kbmap2[] = {
    0, 0, '!', '"', '#', '$', '%', '&', 0, '(', ')', 0, '=', '~', 0, 0,
    'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', '`', '{', 0, 0,
    'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', '+', '*', 0, 0, '}',
    'Z', 'X', 'C', 'V', 'B', 'N', 'M', '<', '>', '?', 0, '*',
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    '7', '8', '9', '-', '4', '5', '6', '+', '1', '2', '3', '0', '.', 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, '|', 0, 0, 0, 0, 0, 0, 0, 0, 0, '|', 0, 0

};




/*
*  Function: initKeyboard
*
*    キーボードを初期化する
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    成功した場合はtrue、失敗した場合はfalseを返します。
*
*/

bool initKeyboard (void){


    keyboard.ctrl        = 0;
    keyboard.alt         = 0;
    keyboard.shift       = 0;
    keyboard.shift       = 0;
    keyboard.CapsLock    = 0;
    keyboard.NumLock     = 0;
    keyboard.ScrollLock  = 0;

    keyboardLED(0);
    return true;
}




/*
*  Function: intKeyboardHandler
*
*    キーボード割り込みのハンドラ
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    なし
*
*/

void intKeyboardHandler (void){

 unsigned char keycode;
 char asccode;


  keycode  = in8(0x60);
  asccode  = convKeycode(keycode);


  if(asccode == 0)
      return;


  Tasks[task.active].keyboard.data[Tasks[task.active].keyboard.writeP] = asccode;

  if(Tasks[task.active].keyboard.writeP > Tasks[task.active].keyboard.size)
    Tasks[task.active].keyboard.writeP = 0;
  else
    Tasks[task.active].keyboard.writeP++;

    return;
}




/*
*  Function: fgetc
*
*    キーボードから一文字読み込む
*
*  Parameters:
*
*    tid - タスクのID
*
*  Returns:
*
*    成功した場合はキーボードから読み込まれた一文字、失敗した場合はEOFを返します。
*
*/

int fgetc (taskid tid){

 char asc;

  if(Tasks[tid].keyboard.readP == Tasks[tid].keyboard.writeP)
    return EOF;

  asc = Tasks[tid].keyboard.data[Tasks[tid].keyboard.readP];

  if(Tasks[tid].keyboard.readP > Tasks[tid].keyboard.size)
    Tasks[tid].keyboard.readP = 0;
  else
    Tasks[tid].keyboard.readP++;

  return asc;
}




/*
*  Function: convKeycode
*
*    キーコードをASCII文字に変換する
*
*  Parameters:
*
*    keycode -  キーコード
*
*  Returns:
*
*    なし
*
*/

char convKeycode (unsigned char keycode){

 char c;


  if(keycode == KEY_CTRL+0x80)
    keyboard.ctrl      = 0;
  if(keycode == KEY_ALT+0x80)
    keyboard.alt       = 0;
  if(keycode == KEY_L_SHIFT+0x80 || keycode == KEY_R_SHIFT+0x80)
    keyboard.shift     = 0;

  if(keycode == KEY_CTRL){
    keyboard.ctrl  = 1; return 0;
  }
  if(keycode == KEY_ALT){
    keyboard.alt   = 1; return 0;
  }
  if(keycode == KEY_L_SHIFT || keycode == KEY_R_SHIFT){
    keyboard.shift = 1; return 0;
  }

  if(keycode == KEY_ENTER)
    return '\n';

  if(keycode == KEY_SPACE)
    return ' ';

  if(keycode == KEY_TAB)
    return '\t';

  if(keycode == KEY_BACKSPACE)
    return '\b';


  if(keycode == KEY_CAPSLOCK){
    keyboardLED(KEY_CAPSLOCK);

    if(keyboard.CapsLock == 1)
      keyboard.CapsLock = 0;
    else
      keyboard.CapsLock = 1;

    return 0;
  }


  if(keycode == KEY_NUMLOCK){
    keyboardLED(KEY_NUMLOCK);

    if(keyboard.NumLock == 1)
      keyboard.NumLock = 0;
    else
      keyboard.NumLock = 1;

    return 0;
  }

  if(keycode == KEY_SCROLLLOCK){
    keyboardLED(KEY_SCROLLLOCK);

    if(keyboard.ScrollLock == 1)
      keyboard.ScrollLock  = 0;
    else
      keyboard.ScrollLock  = 1;

    return 0;
  }

  if(keycode >= 0x80)
    return 0;

  if(keyboard.shift > 0)
    c = kbmap2[keycode];
  else
    c = kbmap1[keycode];

  if(((keyboard.CapsLock == 1 && keyboard.shift == 1) ||
     (keyboard.CapsLock == 0 && keyboard.shift == 0)) &&
     ((c >= 0x41 && c <= 0x5a) || (c >= 0x61 && c <= 0x7a)))
      c += 0x20;

  return c;
}




/*
*  Function: keyboardLED
*
*    キーボードランプを制御する
*
*  Parameters:
*
*    target -  点灯または消灯させるLED
*
*  Returns:
*
*    なし
*
*/

void keyboardLED (unsigned char target){

 char data;


  if(target == 0)
    data = 0;

  if(target == KEY_CAPSLOCK)
    data = (((keyboard.CapsLock)<<2 == 0 ) ? 1<<2 : 0)+((keyboard.NumLock)<<1)+ (keyboard.ScrollLock);
  if(target == KEY_NUMLOCK)
    data = (((keyboard.NumLock)<<1 == 0 ) ?  1<<1 : 0)+((keyboard.CapsLock)<<2)+(keyboard.ScrollLock);
  if(target == KEY_SCROLLLOCK)
    data = (((keyboard.ScrollLock) == 0 ) ?     1 : 0)+((keyboard.CapsLock)<<2)+((keyboard.NumLock)<<1);


 resend1:
  waitKBC(1);
  out8(IOPORT_KBC1, 0xed);
  waitKBC(0);
  if(in8(0x60) == 0xfe)
    goto resend1;

  waitKBC(1);

 resend2:
  out8(IOPORT_KBC1, data);
  waitKBC(0);
  if(in8(0x60) == 0xfe)
    goto resend2;

  return;
}




/*
*  Function: waitKBC
*
*    キーボードコントローラを待つ
*
*  Parameters:
*
*    bit - 監視するビット
*
*  Returns:
*
*    なし
*
*/

void waitKBC (int bit){

  while((in8(IOPORT_KBC2)&0x02) >> bit != 0);
  return;
}

