/*
*    (memory.c)
*/

#include "kernel.h"


/*
*  Function: setPTE
*
*    ページテーブルエントリを設定する
*
*  Parameters:
*
*    ptAddress -  ページディレクトリのアドレス
*    entry     -  エントリの番号
*    addr      -  アドレス
*    mode      -  アクセス権
*    writable  -  書き込み可能か
*    is        -  存在するか
*
*  Returns:
*
*    なし
*
*/

void setPTE (address ptAddress, int entry, unsigned int addr, int mode, int writable, int is){

 address *p;

  p  = (address*) ptAddress+entry;
  *p = (addr&0xfffff000)+(mode<<2|writable<<1|is);

  return;
}




/*
*  Function: initMemory
*
*    メモリを初期化する
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    なし
*
*/

bool initMemory (void){

 volatile address *wp;
 address p = MEMORY_USERAREA_ADDRESS;

  /*
   *  メモリ容量の計算
   */

  disableMemoryCache();

  do{
    p   +=  0x1000;
    wp   =  (address *) p;
    *wp  =  0x55aa55aa;

    if(*wp != 0x55aa55aa)
      break;

  }while(p < 0xffffffff);

  memory.capacity = p;

  enableMemoryCache();


  /*
  *  ページング機構の構築
  */

  //ページエントリをすべてゼロクリア
  for(int i=0; i < 1024; i++)
    *((address *) PAGE_DIRECTORY_ADDRESS+i) = 0;

  //ページディレクトリ
  for(address i=0; i < 1024; i++)
    setPTE(PAGE_DIRECTORY_ADDRESS, i, PAGE_DIRECTORY_ADDRESS+(0x1000*i)+0xf0000, 1, 1, 1);

  //ページテーブル
  for(address i=0; i < 0xfffff; i++)
    setPTE(PAGE_DIRECTORY_ADDRESS+0xf0000, i, (i*0x1000), 1, 1, 1);

  enablePaging((address) (PAGE_DIRECTORY_ADDRESS)&0xfffff000);


  return true;
}




/*
*  Function: kmalloc
*
*    メモリを割り当てる
*
*  Parameters:
*
*    size - 取得するメモリのサイズ
*
*  Returns:
*
*    成功した場合はそのメモリのアドレス、失敗した場合はNULLを返します。
*
*/

void *kmalloc (unsigned int size){

 int flag;
 unsigned int blockSize , i , j;


  flag = 0;
  blockSize = (size/MEMORY_BLOCK_SIZE)+1;

  for(i=0;; i++){
    if(memory.block[i].status == MEMORY_UNUSED){
      for(j=0; (memory.block[i].status == MEMORY_UNUSED) && (j <= blockSize); i++, j++){
         if(j == blockSize){
           flag = 1;
           goto ret;
          }
       }
    }
  }

 ret:

  if(!flag)
    return NULL;

  for(j = i; j < (blockSize+i); j++)
    memory.block[j].status = MEMORY_USING;

  return (void *) (i*MEMORY_BLOCK_SIZE)+MEMORY_USERAREA_ADDRESS;
}




/*
*  Function: kfree
*
*    メモリを開放する
*
*  Parameters:
*
*    p   - メモリへのポインタ
*    tid - (利用していません)
*
*  Returns:
*
*    成功した場合はそのメモリのアドレス、失敗した場合はNULLを返します。
*
*/

void kfree (void *p , taskid tid){

  for(unsigned int i = ((unsigned int) p-MEMORY_USERAREA_ADDRESS)/MEMORY_BLOCK_SIZE; memory.block[i].status == 1; i++)
    memory.block[i].status = MEMORY_UNUSED;

  return;
}

