/*
 *   (task.c)
 */

#include "kernel.h"

/*
*  Function: initTask
*
*    タスク管理を初期化する
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    成功した場合はtrue、失敗した場合はfalseを返します。
*
*/

bool initTask (void){

 TSS *tss = (TSS *) TSS_ADDRESS;

  task.running     = 0;
  task.sleepingNum = 0;
  task.runningNum  = 0;

  for(int i = 0; i < TASK_MAX; i++)
    Tasks[i].status = TASK_UNUSED;

  tss->ss0    = GDT_DATA_SELECTOR;
  tss->esp0   = STACK_ADDRESS;

  ltr(GDT_TSS_SELECTOR);

  return true;
}




/*
*  Function: allocTask
*
*    使用可能なタスクのIDを取得する
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    成功した場合は使用可能なタスクのID、失敗した場合は0を返します。
*
*/

taskid allocTask (void){

 taskid tid;

  for(tid=TASKID_START; (Tasks[tid].status != TASK_UNUSED) && (tid < TASKID_END); tid++);

  if(tid != TASKID_END)
    return tid;
  else
    return 0;

}





/*
*  Function: freeTask
*
*    タスクを放棄する
*
*  Parameters:
*
*    i -  タスクのID
*
*  Returns:
*
*    なし
*
*/

void freeTask (taskid tid){


  Tasks[tid].status = TASK_UNUSED;
  task.runningNum--;

  return;
}





/*
*  Function: startTask
*
*    タスクを実行する
*
*  Parameters:
*
*    eip - オフセット
*
*  Returns:
*
*    なし
*
*/

bool startTask (address eip){

 taskid tid;

  for(tid=TASKID_START; (Tasks[tid].status != TASK_UNUSED) && (tid < TASKID_END); tid++);

  if(tid == TASKID_END)
    return false;


  Tasks[tid].priority         = TASK_DEFAULT_PRIORITY;
  Tasks[tid].regs.eip         = eip;
  Tasks[tid].regs.eflags      = TASK_DEFAULT_EFLAGS;
  Tasks[tid].cr3              = PAGE_DIRECTORY_ADDRESS;
  Tasks[tid].regs.cs          = GDT_USERCODE_SELECTOR+3;
  Tasks[tid].regs.ds          = GDT_USERDATA_SELECTOR+3;
  Tasks[tid].regs.es          = GDT_USERDATA_SELECTOR+3;
  Tasks[tid].regs.fs          = GDT_USERDATA_SELECTOR+3;
  Tasks[tid].regs.gs          = GDT_USERDATA_SELECTOR+3;
  Tasks[tid].regs.ss          = GDT_USERDATA_SELECTOR+3;
  Tasks[tid].regs.esp         = 0x1000;

  Tasks[tid].regs.eax         = 0;
  Tasks[tid].regs.ebx         = 0;
  Tasks[tid].regs.ecx         = 0;
  Tasks[tid].regs.edx         = 0;
  Tasks[tid].regs.esi         = 0;
  Tasks[tid].regs.edi         = 0;
  Tasks[tid].regs.ebp         = 0;

  task.runningNum++;
 
  Tasks[tid].status = TASK_RUNNING;
  return true;
}





/*
*  Function: switchTask
*
*    タスクを切り替える
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    なし
*
*/

void switchTask (void){

 taskid i;

  if(task.runningNum == 0)
    return;

  for(i=task.running+1; (Tasks[i].status != TASK_RUNNING) && (i != task.running); i++){
    if(i == TASKID_END)
      i = TASKID_START-1;
  }

  if(task.running != 0)
    asm_saveTask(&Tasks[task.running].regs);

  task.running = i;
  setTimer(TIMER_TASK, Tasks[i].priority);

  asm_runTask(&Tasks[i].regs, PAGE_DIRECTORY_ADDRESS);
}

